"BlogFace - simple webblog product"

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Globals import DTMLFile
from OFS.Folder import Folder
from Products.ZCatalog.ZCatalog import ZCatalog
from DateTime import DateTime
#from OFS.Cache import Cacheable
import re
import string
from datetools import seqToDate, dateToSeq
            
def addBlogFace(self, id, title, newCatalog = None, REQUEST = None):
    "add BlogFace instance"
    ob = BlogFace(id, newCatalog)
    ob.title = str(title)
    id = self._setObject(id, ob)
    if REQUEST: 
        return self.manage_main(self, REQUEST)

manage_addBlogFace = DTMLFile('dtml/BlogFaceAdd', globals())

class BlogFace(Folder):
    "Container and manager class for weblog items"

    meta_type='BlogFace'

    security = ClassSecurityInfo()

    manage_blogForm = DTMLFile('dtml/main', globals())

    manage_options = ({'label' : 'Main', 'action' : 'manage_blogForm',
                      'help' : ('BlogFace', 'BlogFace_main.stx')},) + \
                      Folder.manage_options

    # entry attributes to catalog, if we build a default catalog
    catAttrs = (('title', 'FieldIndex'), ('date', 'FieldIndex'),
                ('document_src', 'TextIndex'))
    # catalog index to get the date from
    dateIndex = 'date'

    # entries displayed per page
    entriesDisplayed = 10
    # resolution for entry URLs - 1=year, 2=month, 3=day, 4=hour...
    entryRes = 5

    # dict for turing traversal params into search refine arguments
    # keys are params in the URL, values are 2-tuples for the
    # catalog search dict - criterion, value.
    # keys cannot be intable, since those are turned to date params
    # keys that match subobject ids will shadow traversing to subobjects
    # this is set in __init__
    #traverseParamOpts = {}

    def __init__(self, id, newCatalog = 1):
        self.id = id
        # why does this need to be set here, and not in the class?
        # perhaps because dicts aren't autopersistent?
        self.traverseParamOpts = {}        
        if newCatalog:
            self.initCatalog()

    def initCatalog(self):
        "init the default blog catalog"
        if not hasattr(self, 'Catalog'):
            self._setObject('Catalog',
                            ZCatalog('Catalog', title='', container=self))
        self.catalogPath = "Catalog"
        # add indices, tables if we don't have them yet
        schema = self.getCatalog().schema()
        indexes = self.getCatalog().indexes()
        for at in self.catAttrs:
            if not at[0] in schema:
                self.getCatalog().manage_addColumn(at[0])
            if not at[0] in indexes:
                self.getCatalog().manage_addIndex(at[0], at[1])

    def getCatalog(self):
        "Return the catalog used."
        return self.unrestrictedTraverse(self.catalogPath)

    security.declareProtected('View management screens',
                              'getTraverseParamOpts') 
    def getTraverseParamOpts(self):
        """
        Return traverse paramater options as items.
        This is to make it easier to handle in DTML.
        """
        return self.traverseParamOpts.items()

    security.declarePublic('searchForward')
    def searchForward(self, date, params = ()):
        "Return the results of a forward search of the catalog."
        dict = {}
        dict['sort_on'] = self.dateIndex
        dict[self.dateIndex] = date
        dict[self.dateIndex + '_usage'] = 'range:max'
        dict['sort_order'] = 'reverse'
        for param in params:
            dict[param[0]] = param[1]
        return self.getCatalog().searchResults(dict)

    security.declarePublic('searchReverse')
    def searchReverse(self, date, params = ()):
        "Return the results of a backward search of the catalog."
        dict = {}
        dict['sort_on'] = self.dateIndex
        dict[self.dateIndex] = date
        dict[self.dateIndex + '_usage'] = 'range:min'
        dict['sort_order'] = 'ascending'
        for param in params:
            dict[param[0]] = param[1]
        return self.getCatalog().searchResults(dict)

    def __bobo_traverse__(self, request, name):
        """
        Consume trailing URL path and turn into arguments so long as
        they are convertable to ints.  Otherwise, traverse normally with
        collected arguments.  Note that this shadows subobjects with IDs
        that are convertable to ints.
        """
        if re.match('^[0-9]+$', name):
            if not request.has_key('traverse_date'):
                request['traverse_date'] = []
            (request['traverse_date']).append(name)
        elif name in self.traverseParamOpts.keys():
            if not request.has_key('traverse_params'):
                request['traverse_params'] = []
            (request['traverse_params']).append(name)
        else: # normal traverse
            return getattr(self, name)
        return self

# these functions made it easier to add an entry to the BlogFace container
# by only allowing entry types in.
#      def _isAllowedMetaType(self, meta_type):
#          """
#          Return true if meta_type, a dict of the type returned by an
#          all_meta_type call, has a name that is in self_meta_type_names.
#          Helper function for all_meta_types.
#          """
#          # we just do this to get around lambda scope - is this just dense?
#          return meta_type['name'] in self.entry_meta_type_names

#      def all_meta_types(self, interfaces = None):
#          """
#          Return a list of meta types that the user is allowed to add as
#          subobjects.  This is used to only allow certain entry types to
#          be added.
#          """
#          return filter(self._isAllowedMetaType,
#                        Folder.all_meta_types(self, interfaces))

    security.declareProtected('View management screens', 'manage_blog') 
    def manage_blog(self, title, entries, entryRes, dateIndex,
                    catalogPath, delSearch = None, search = None,
                    REQUEST = None):
        "main management method"
        self.title = str(title)
        self.entriesDisplayed = int(entries)
        self.entryRes = int(entryRes)
        if catalogPath:
            self.catalogPath = catalogPath
        if dateIndex:
            self.dateIndex = dateIndex
        if delSearch:
            for i in delSearch:
                del self.traverseParamOpts[i]
                self._p_changed = 1
        if search:
            # XXX can't have a search criteria w/o a path param
            if (search.get('param', None) and search.get('term', None) and
                search.get('crit', None)):
                self.traverseParamOpts[search.param] = (
                    search.term, search.crit)
                self._p_changed = 1
        if REQUEST:
            message = "Saved changes."
            return self.manage_blogForm(self, REQUEST,
                                        management_view="Edit",
                                        manage_tabs_message=message)

    security.declarePublic('hasEntryDay')
    def hasEntryDay(self, date):
        """
        Return true if there is an entry between the time of the DateTime date
        and the end of that day.
        """
        # note we don't use the URL params here
        dict = {}
        dict['sort_on'] = self.dateIndex
        dict[self.dateIndex] = [date, date.latestTime()]
        dict[self.dateIndex + '_usage'] = 'range:min:max'
        #dict['sort_order'] = 'ascending'
        return len(self.getCatalog().searchResults(dict)) > 0
        
    security.declarePublic('dateURL')
    def dateURL(self, date, resolution = None):
        """
        Return a URL for the page for date.  Resolution describes
        how many stages past self the path should have - year, month, day...
        """
        if not resolution:
            resolution = self.entryRes
        return self.absolute_url() + '/' + string.join(
            dateToSeq(date, resolution), '/')

    # Is this efficient?  Search results are LazyMaps,
    # so we shouldn't be grumbling through the entire catalog
    # if we only use the front few entries
    def getEntrySeq(self, date, params = ()):
        """
        Return a tuple containing the sequence of latest entries earlier
        than date, the URL of the previous
        sequence or None, and the URL of the next sequence or None.
        """
        searchOut = self.searchForward(date, params)
        lastOut = self.entriesDisplayed
        # add to return list so long as they're on the same day
        while (len(searchOut) > lastOut and
               dateToSeq(searchOut[lastOut-1].date, 3) ==
               dateToSeq(searchOut[lastOut].date, 3)):
            lastOut += 1
        # now index lastOut is a later day than lastOut-1, if it exists.
        if len(searchOut) > lastOut:
            # earlyURL is the URL for that prev day
            earlyURL = self.dateURL(searchOut[lastOut].date, 3)
        else:
            earlyURL = None
        revSearchOut = self.searchReverse(date, params)
        if len(revSearchOut) > 1:
            # lateURL is the URL for the next day
            lateURL = self.dateURL(revSearchOut[0].date, 3)
        else:
            lateURL = None
        return (searchOut[0:lastOut], earlyURL, lateURL)

    security.declarePublic('requestDate')
    def requestDate(self, REQUEST):
        "Return a DateTime for the date passed by the REQUEST date parameters."
        if not REQUEST.has_key('traverse_date'):
            REQUEST['traverse_date'] = []
        return seqToDate(REQUEST.traverse_date)    

    security.declarePublic('requestParams')
    def requestParams(self, REQUEST):
        "Return the search params corresponding to the REQUEST parameters."
        searchParams = []
        if REQUEST.has_key('traverse_params'):
            for param in REQUEST['traverse_params']:
                searchParams.append(self.traverseParamOpts[param])
        return searchParams
    
    security.declarePublic('index_html')
    def index_html(self, REQUEST, RESPONSE):
        """
        Return the index page with blog entries searched according to
        args collected by __bobo_traverse__, or return error page on raise.
        """
        date = self.requestDate(REQUEST)
        searchParams = self.requestParams(REQUEST)
        # put these in REQUEST, otherwise DTML pages called from the
        # index page don't get them.  I don't think this is necessary,
        # I might be missing something...
        REQUEST.set('blogDate', date)
        REQUEST.set('searchParams', searchParams)        
        try:
            (entryList, earlyURL, lateURL) = self.getEntrySeq(date,
                                                              searchParams)
        except:
            return self.URLFmtError(self, REQUEST)            
        REQUEST.set('entryList', entryList)
        REQUEST.set('lateURL', lateURL)
        REQUEST.set('earlyURL', earlyURL)
        return self.index_htmlPage(self, REQUEST, RESPONSE)
    
    index_htmlPage = DTMLFile('dtml/index', globals())

    security.declarePublic('entries')
    entries = DTMLFile('dtml/entries', globals())

    security.declarePublic('blogCalendar')
    blogCalendar = DTMLFile('dtml/blogCalendar', globals())

    security.declarePublic('URLFmtError')
    URLFmtError = DTMLFile('dtml/URLFmtError', globals())

InitializeClass(BlogFace)
