"""
STXDocumentEntry - weblog entry product based on a STXDocument.
This is just a STXDocument that is a PropertyManager and which sets
a 'date' property on creation.  It also has CMF style attribute arguments
for PUT (FTP/DAV) creation/editing.  These arguments set existing properties.
"""

from Globals import InitializeClass
from Globals import DTMLFile
from Products.ZCatalog.CatalogAwareness import CatalogAware
from Products.STXDocument.STXDocument import STXDocument
from OFS.PropertyManager import PropertyManager
from DateTime import DateTime
from urllib import quote
from Products.CMFDefault.utils import parseHeadersBody

def initialize(registrar):
    registrar.registerClass(
        STXDocumentEntry,
        permission = 'Add Documents, Images, and Files',
        # note that this path is from the BlogFace directory, since this
        # is run from its __init__.py
        icon = 'www/STXDocumentEntry.gif',
        constructors = (addSTXDocumentEntryForm,
                        addSTXDocumentEntry),
        )
    registrar.registerHelp()
    registrar.registerHelpTitle('SimpleEntry Help')

addSTXDocumentEntryForm = DTMLFile('dtml/addSTXDocumentEntryForm', globals())

# this is just manage_addSTXDocument.  Date is gotten from REQUEST.
def addSTXDocumentEntry(self, id, title='', doc='', REQUEST=None):
    "add an STXDocumentEntry"

    id=str(id)
    title=str(title)

    d=STXDocumentEntry(id, title, doc, REQUEST)
    self._setObject(id, d)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

class STXDocumentEntry(CatalogAware, PropertyManager, STXDocument):
    """
    Subclass of STXDocument, adds properties, automatic date property
    on creation, CMF text-style PUT headers.
    """
    meta_type = "STXDocumentEntry"

    manage_options = STXDocument.manage_options + \
                     PropertyManager.manage_options

    def __init__(self, id, title='', doc='', REQUEST={}):
        STXDocumentEntry.inheritedAttribute('__init__')(
            self, id, title, doc, REQUEST)
        self._init_date(REQUEST.get('date', None))


    def _init_date(self, date = None):
        "Add date property.  Meant to be called from __init__."
        if not date:
            date = DateTime()
        self.manage_addProperty('date', date, 'date')
    
    # this happens before the _setobject
    def PUT(self, REQUEST, RESPONSE):
        "Handle HTTP PUT requests.  Set properties from headers if there."
        # strip out the headers CMF style, put body back in
        headers, body = parseHeadersBody(REQUEST.get('BODY', ''))
        REQUEST.set('BODY', body)
        RESPONSE = STXDocumentEntry.inheritedAttribute('PUT')(
            self, REQUEST, RESPONSE)
        # set any properties from the headers
        apply(self.manage_changeProperties, (), headers)
        return RESPONSE

InitializeClass(STXDocumentEntry)
